<?php

namespace Drupal\yasm_charts\Services;

use Drupal\charts\Services\ChartsSettingsServiceInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Component\Uuid\UuidInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Yasm charts build helper class.
 */
class YasmChartsBuilder implements YasmChartsBuilderInterface {

  use StringTranslationTrait;

  /**
   * The charts settings.
   *
   * @var \Drupal\charts\Services\ChartsSettingsServiceInterface
   */
  protected $chartSettings;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The UUID service.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidService;

  /**
   * {@inheritdoc}
   */
  public function discoverCharts($build, $settings) {
    if (!empty($this->chartSettings['library'])) {
      foreach ($build as $key => $element) {
        if ('#yasm_chart' === $key) {
          $chart = $this->applyChartSettings($build['#yasm_chart'], $build['yasm_table'], $settings);
          if (is_array($chart)) {
            $build['yasm_chart'] = $chart;
          }
        }
        elseif (is_array($element)) {
          // Recursive call through array depths.
          $build[$key] = $this->discoverCharts($element, $settings);
        }
      }
    }
    else {
      $this->messenger->addError($this->t('To display charts first needs to configure <a href="@link">charts settings</a>.', [
        '@link' => '/admin/config/content/charts',
      ]));
    }

    return $build;
  }

  /**
   * Build yasm chart.
   */
  private function buildChart($chart, $settings) {
    // Skip data settings.
    $settings['skip_left'] = isset($settings['skip_left']) ? $settings['skip_left'] : 1;
    $settings['skip_right'] = isset($settings['skip_right']) ? $settings['skip_right'] : 0;
    $settings['skip_top'] = isset($settings['skip_top']) ? $settings['skip_top'] : 0;
    // Label settings.
    $settings['label'] = isset($settings['label']) ? $settings['label'] : '';
    $settings['label_position'] = isset($settings['label_position']) ? $settings['label_position'] : 1;
    // Options.
    $settings['options'] += $this->chartSettings;
    if (isset($chart['#title'])) {
      $settings['options']['title'] = $chart['#title'];
    }
    $settings['options']['type'] = isset($settings['type']) ? $settings['type'] : 'line';
    // Series data.
    $seriesData = $this->getChartSeries($chart['#rows'], $settings);
    // Categories.
    if ($settings['options']['type'] == 'pie' && count($seriesData) === 1) {
      $categories = [$seriesData[0]['name']];
    }
    else {
      $categories = $this->getChartCategories($chart['#header'], $settings);
    }
    // Chart Unique ID.
    if (!isset($settings['id'])) {
      $settings['id'] = $this->uuidService->generate();
    }

    return [
      [
        '#theme' => 'yasm_chart',
        '#library' => (string) $this->chartSettings['library'],
        '#categories' => $categories,
        '#seriesData' => $seriesData,
        '#options' => $settings['options'],
        '#id' => 'chart-' . $settings['id'],
        '#override' => isset($settings['override']) ? $settings['override'] : [],
      ],
    ];
  }

  /**
   * Get chart array series.
   */
  private function getChartSeries($rows, $settings) {
    $series = [];
    $i = 0;
    if ($settings['skip_top'] > 0) {
      $rows = array_slice($rows, $settings['skip_top']);
    }
    foreach ($rows as $row) {
      $row = (is_array($row) && isset($row['data'])) ? $row['data'] : $row;
      $label = $this->getChartLabel($row, $settings['label'], $settings['label_position']);
      $row = $this->skipArray($row, $settings['skip_left'], $settings['skip_right']);
      $items = [];
      foreach ($row as $col) {
        $item = isset($col['data']) ? $col['data'] : $col;
        $items[] = (int) $item;
      }

      $series[] = [
        'type'  => $settings['options']['type'],
        'name'  => (string) $label,
        'color' => $this->getChartColor($i),
        'data'  => $items,
      ];
      $i++;
    }

    return $series;
  }

  /**
   * Get chart categories for X axis.
   */
  private function getChartCategories($labels, $settings) {
    $categories = $this->skipArray($labels, $settings['skip_left'], $settings['skip_right']);

    return array_values($categories);
  }

  /**
   * Get colors AA colors for charts.
   */
  private function getChartColor($i) {
    $colors = [
      0 => '#205CB7',
      1 => '#815D51',
      2 => '#5EB15A',
      3 => '#9A38B2',
      4 => '#00BFD5',
      5 => '#009A8E',
      6 => '#FFC21C',
      7 => '#009CF2',
      8 => '#FF9B0F',
      9 => '#F84C3E',
      10 => '#99C455',
      11 => '#EC2C69',
    ];

    if (isset($colors[$i])) {
      $color = $colors[$i];
    }
    else {
      $color = $colors[rand(0, count($colors) - 1)];
    }

    return $color;
  }

  /**
   * Skip some elemenys of array.
   */
  private function skipArray($array, $skip_left = 1, $skip_right = 0) {
    if ($skip_left > 0) {
      $array = array_slice($array, $skip_left);
    }
    if ($skip_right > 0) {
      $array = array_slice($array, 0, count($array) - $skip_right);
    }

    return $array;
  }

  /**
   * Get chart label from skiped elements.
   */
  private function getChartLabel($row, $label, $label_position) {
    if (empty($label) && $label_position > 0) {
      $col = array_slice($row, $label_position - 1, 1);
      $col = reset($col);
      $col = (is_array($col) && isset($col['data'])) ? $col['data'] : $col;
      $label = (is_array($col)) ? reset($col) : $col;
      $label = (string) $label;
    }

    return $label;
  }

  /**
   * Appply chart settings and build the chart array.
   */
  private function applyChartSettings($key, $table, $settings) {
    if (isset($settings[$key])) {
      return $this->buildChart($table, $settings[$key]);
    }

    return $key;
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(ChartsSettingsServiceInterface $chartSettings, MessengerInterface $messenger, UuidInterface $uuidService) {
    $this->chartSettings = $chartSettings->getChartsSettings();
    $this->messenger = $messenger;
    $this->uuidService = $uuidService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('charts.settings'),
      $container->get('messenger'),
      $container->get('uuid')
    );
  }

}
